/*
 * main.cpp
 *
 *  Created on: Oct 30, 2008
 *      Author: qwan
 */

//#define DEBUG;
#include "skyline.h"
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <math.h>

#define BILLION  1000000000L;

int tableNum;

PtsSpace* tables;
vector<int>* rule;
bool indexed = true;
bool calbnl = true;
vector<int*>* result;
stack<int*>* mempool;

float userTime_read, sysTime_read;
float userTime_prep, sysTime_prep;
float userTime_algo, sysTime_algo;
float userTime_bnl, sysTime_bnl;
float userTime_overall, sysTime_overall;
struct rusage myTime_program_start, myTime_read_end, myTime_prep_end,
		myTime_bnl_end, myTime_algo_end;

void calculateExecutionTime(struct rusage *myTimeStart,
		struct rusage *myTimeEnd, float *userTime, float *sysTime) {
	(*userTime) = ((float) (myTimeEnd->ru_utime.tv_sec
			- myTimeStart->ru_utime.tv_sec))
			+ ((float) (myTimeEnd->ru_utime.tv_usec
					- myTimeStart->ru_utime.tv_usec)) * 1e-6;
	(*sysTime) = ((float) (myTimeEnd->ru_stime.tv_sec
			- myTimeStart->ru_stime.tv_sec))
			+ ((float) (myTimeEnd->ru_stime.tv_usec
					- myTimeStart->ru_stime.tv_usec)) * 1e-6;

}

void printTimeFile() {

	FILE *filePtr;
	//	char *filename = "time";

	filePtr = (FILE *) fopen("time_bs", "w");
	if (filePtr == NULL) {
		printf("File cannot be opened in printTime!\n");
		exit(0);
	}

	// output execution time
	calculateExecutionTime(&myTime_program_start, &myTime_read_end,
			&userTime_read, &sysTime_read);
	calculateExecutionTime(&myTime_read_end, &myTime_prep_end, &userTime_prep,
			&sysTime_prep);
	calculateExecutionTime(&myTime_prep_end, &myTime_algo_end, &userTime_algo,
			&sysTime_algo);
	calculateExecutionTime(&myTime_algo_end, &myTime_bnl_end, &userTime_bnl,
			&sysTime_bnl);
	calculateExecutionTime(&myTime_program_start, &myTime_bnl_end,
			&userTime_overall, &sysTime_overall);

	// reading time
	fprintf(filePtr, "%.10f %.10f\n", userTime_read, sysTime_read);

	// sorting time
	fprintf(filePtr, "%.10f %.10f\n", userTime_prep, sysTime_prep);

	// writing time
	fprintf(filePtr, "%.10f %.10f\n", userTime_algo, sysTime_algo);

	// writing time
	fprintf(filePtr, "%.10f %.10f\n", userTime_bnl, sysTime_bnl);

	// overall time
	fprintf(filePtr, "%.10f %.10f\n", userTime_overall, sysTime_overall);

	fclose(filePtr);
}

int load() {
	//open file
	FILE * pFile;
	if ((pFile = fopen("./config", "r")) == NULL) {
		printf("reading error!!!");
		//exit(0);
		return 0;
	}
	//get number of tables
	fscanf(pFile, "%d", &tableNum);

	//Object "tables" stores all table in this program
	//load config for tables
	tables = new PtsSpace[tableNum];
	tables[0].isPackage = true;

	char* fileName = new char[100];
	FILE** fArray = new FILE*[tableNum];

	int tup = 0;

	for (int i = 0; i < tableNum; i++) {
		sprintf(fileName, "table%d_skyline", i);
		if ((fArray[i] = fopen(fileName, "r")) == NULL) {
			sprintf(fileName, "table%d", i);
			if ((fArray[i] = fopen(fileName, "r")) == NULL) {
				printf("data reading error!!!");
				return 0;
			}
		} else {
			fscanf(fArray[i], "%d\n", &tup);
			calbnl = false;
		}

	}

	int d, t, n;

	fscanf(pFile, "%d%d%d", &d, &t, &n);
	if (tup == 0) {
		tables[0].init(d, t, n);
	} else {
		tables[0].init(d, tup, n);
	}

	for (int i = 1; i < tableNum; i++) {
		fscanf(pFile, "%d%d%d", &d, &t, &n);
		tables[i].init(d, t, n);
	}

	//load rules
	int dim = tables[0].dimNum;
	rule = new vector<int> [dim];

	for (int i = 0; i < dim; i++) {
		fscanf(pFile, "%d", &d);
		rule[i].resize(2 * d);
	}

	//transform the rule for future convenience
	// n is table index and n+1 is dim index
	for (int i = 0; i < dim; i++) {
		for (int j = 0; j < rule[i].size(); j++) {
			fscanf(pFile, "%d", &rule[i][j]);
			if (j % 2 == 1) {
				tables[rule[i][j - 1]].f[rule[i][j]] = i;
			}
		}
	}

	fclose(pFile);

	//load table data

	printf("start loading.....\n");

	for (int l = 0; l < tableNum; l++) { //l-th table
		tables[l].getdata(fArray[l]);
		fclose(fArray[l]);
	}

	//tables[0].show();

	return 1;
}

int dominate(const int* v1, const int* v2, int n) {
	skyline::cmp_countL++;
	bool v1grt = false, v2grt = false;

	for (int i = 0; i < n; i++) {
		if (v1[i] < v2[i])
			v1grt = true;
		if (v2[i] < v1[i])
			v2grt = true;

	}

	if (v1grt && v2grt) {
		return 2; //no dominate relation
	} else if (v1grt) {
		return 1; //v2 dominate v1
	} else if (v2grt) {
		return -1; //v1 dominate v2
	} else {
		return 0; //v1 = v2
	}
}
;

void recurPortal(int depth, vector<int> &s) {

	if (depth == 1) {
		for (int i = 0; i < tables[1].tupleNum; i++) {
			s.push_back(i);
			recurPortal(depth + 1, s);

			int done = (int) ((1.0 * i) / tables[1].tupleNum * 100);
			printf("under calculation, %d percentages done\n", done);

			s.pop_back();
		}

		printf("under calculation, 100 percentages done!!!\n");

	} else if (depth < tableNum) {
		for (int i = 0; i < tables[depth].tupleNum; i++) {
			s.push_back(i);
			recurPortal(depth + 1, s);
			s.pop_back();
		}
	} else {
		int* t;
		//calculate the vector
		if (mempool->empty()) {
			t = new int[tables[0].dimNum];
		} else {
			t = mempool->top();
			mempool->pop();
		}

		memset(t, 0, tables[0].dimNum * sizeof(int));

		for (int i = 0; i < tables[0].dimNum; i++) {
			t[i] = 0;
		}

		for (int i = 1; i < tableNum; i++) {
			int* pt = tables[i].v[s[i - 1]];
			int* f = tables[i].f;
			for (int j = 0; j < tables[i].dimNum; j++) {
				t[f[j]] += pt[j];
			}
		}

		if (tables[0].rect_checking(t)) {
			//result->push_back(t);
			skyline::skyline_count++;
			mempool->push(t);
		} else {
			mempool->push(t);
		}

		while (mempool->size() > 10) {
			delete[] mempool->top();
			mempool->pop();
		}

	}

}

void timetofile(struct timespec stop, struct timespec start, FILE * f) {
	double accum = (stop.tv_sec - start.tv_sec) + (double) (stop.tv_nsec
			- start.tv_nsec) / (double) BILLION;
	fprintf(f, "\n%lf", accum);
}

int main(int argc, char** argv) {

	result = new vector<int*> ;
	mempool = new stack<int*> ;

	if (argc != 2) {
		return 0;
	}

	int chose = atoi(argv[1]);

	getrusage(RUSAGE_SELF, &myTime_program_start);

	FILE* f = fopen("stat_bs", "w+");

	if (load() == 0)
		return 0;

	printf("done with loading....\n");

	getrusage(RUSAGE_SELF, &myTime_read_end);

	long mt0 = 1;
	for (int i = 0; i < tableNum; i++) {
		//fprintf(f,"%ld,", tables[i].tupleNum);
		if (i != 0) {
			mt0 *= tables[i].tupleNum;
		}
	}

	if (calbnl) {
		for (int i = 0; i < tableNum; i++) {
			tables[i].bnl();
		}

		tables[0].tofile("table0_skyline");
	} else {
		for (int i = 1; i < tableNum; i++) {
			tables[i].bnl();
		}
	}

	if (chose == 1) {
		tables[0].build_index();
	} else if (chose == 2) {
		tables[0].build_index();
	}

	long mt1 = 1;
	for (int i = 0; i < tableNum; i++) {
		//fprintf(f,"%ld,", tables[i].tupleNum);
		if (i != 0) {
			mt1 *= tables[i].tupleNum;
		}
	}

	getrusage(RUSAGE_SELF, &myTime_prep_end);

	vector<int> s(0);
	recurPortal(1, s);

	getrusage(RUSAGE_SELF, &myTime_algo_end);

	//	skyline::memcost += result->size() * tables[0].dimNum * sizeof(int);

	getrusage(RUSAGE_SELF, &myTime_bnl_end);

	fprintf(f, "%ld,", skyline::skyline_count);
	fprintf(f, "%ld,", PtsSpace::totalAsked);
	fprintf(f, "%ld,", skyline::memcost);
	fprintf(f, "%ld,", mt0);
	fprintf(f, "%ld", mt1);

	printTimeFile();

	fclose(f);

	return EXIT_SUCCESS;
}
