#ifndef SKYLINE_H_
#define SKYLINE_H_

#include "common.h"
#include "space.h"
#include "formula.h"

#define _MAXIMUM 65535
#define _ITER_THD 10

using namespace std;

CSetformula* formula_intersection(CSetformula* f1, CSetformula* f2);
CFormulaNode* node_intersection(CFormulaNode* set1, CFormulaNode* set2);

class skyline {
public:
	//
	CFormulaNode** matrix; //[no_package][no_table]
	CSetformula** cache; //each point to a node
	CalTree* ct;
	int iter_thd;

	int tableNum;

	PtsSpace** tables;
	vector<int>* rule;
	Pt_Collection** CR;
	Pt_Collection** NCR;

	int* extend;

	skyline() {
		//getdata();
		iter_thd = 5;
	}

	~skyline() {
		freemem();
	}

	void run();
	void freemem();
	void freemem2();
	int compare(int* x, int* y, int n);
	void printStruc();
	void addto_vector(int* v, int tabNo, int ptNo);
	void calculateCR();
	void compute_skyline(CalNode* ptr);
	void compute_skyline(int n);
	void algo_portal();
	void ordering();
	void transform();
	void preprocessing();
	void rule_mapping(int* map);
	void selectDim();
	void calCache();

	void setAlgo(PtsSpace* p, vector<PtsSpace*> vp, vector<int>* r, int tn);
	void setAlgo2(PtsSpace* tables, vector<int>* r, int tn);

	static unsigned long cmp_countH;
	static unsigned long cmp_countL;
	static unsigned long rectquery_count;
	static unsigned long skyline_count;
	static unsigned long pruned;
	static unsigned long memcost;
	static unsigned long memmax;
};

#endif
