/*
 * caltree.h
 *
 *  Created on: Jan 14, 2009
 *      Author: qwan
 */

#ifndef CALTREE_H_
#define CALTREE_H_

#include "common.h"
#include "formula.h"

class CalNode {
public:
	list<CalNode*> children_list;
	list<int> pts;
	set<int>* parent_path;
	CSetformula* formula;
	CalNode* parent;
	bool sharedformula;

	static int cnt;

	void show() {
		printf("===================================\n");

		printf("\nhave children %d", children_list.size());
		printf("\ncontains points %d:\n", pts.size());
		list<int>::iterator i;

		for (i = pts.begin(); i != pts.end(); ++i) {
			printf("%d ", *i);
		}

		printf("\n");

		if (parent_path != NULL) {
			printf("path contains packages points %d:\n", parent_path->size());
			set<int>::iterator j;
			for (j = parent_path->begin(); j != parent_path->end(); ++j) {
				printf("%d ", *j);
			}
		}

		printf("\n");
	}
	;

	CalNode() {
		parent_path = NULL;
		formula = NULL;
		parent = NULL;
		sharedformula = false;
		CalNode::cnt++;
	}
	;

	~CalNode() {
		CalNode::cnt--;
		if (parent_path != NULL) {
			parent_path->clear();
			delete parent_path;
		}
		if (formula != NULL && !sharedformula) {
			delete formula;
		}
	}
	;

	long count() {
		long size = 0;

		//size += sizeof(*parent_path);

		size += sizeof(children_list) * children_list.size();
		size += sizeof(pts) * pts.size();

		if (parent_path != NULL) {
			size += sizeof(*parent_path) * parent_path->size();
		}

		size += sizeof(CalNode*);
		if (!sharedformula)
			size += formula->count();

		return size;
	}

};

class CalTree {
public:
	CalNode* root;
	//list<CalNode* >* ns;

	void insert(int pts_id, Pt_Collection* region);
	void tmpFreeTree();
	CalTree() {
		root = new CalNode;
		//		ns = new stack<CalNode* >;
	}
	;

	~CalTree() {
		stack<CalNode*>* nodeStack = new stack<CalNode*> ;
		//stack<list<CalNode* >::iterator > itStack;
		nodeStack->push(root);
		//itStack.push(root->children_list.begin());

		while (!nodeStack->empty()) {
			CalNode* tp = nodeStack->top();

			nodeStack->pop();
			list<CalNode*>::iterator i;
			for (i = tp->children_list.begin(); i != tp->children_list.end(); i++) {
				nodeStack->push(*i);
			}

			delete tp;

		}
		delete nodeStack;
	}

	void show() {
		stack<CalNode*> nodeStack;
		nodeStack.push(root);

		while (!nodeStack.empty()) {
			CalNode* tp = nodeStack.top();
			nodeStack.pop();
			list<CalNode*>::reverse_iterator i;
			for (i = tp->children_list.rbegin(); i != tp->children_list.rend(); ++i) {
				nodeStack.push(*i);
			}
			tp->show();
		}

	}

	long count() {
		stack<CalNode*> nodeStack;
		nodeStack.push(root);

		long size = 0;

		while (!nodeStack.empty()) {
			CalNode* tp = nodeStack.top();
			nodeStack.pop();
			list<CalNode*>::reverse_iterator i;
			for (i = tp->children_list.rbegin(); i != tp->children_list.rend(); ++i) {
				nodeStack.push(*i);
			}

			if (tp != root) {
				size += tp->count();
			}

		}

		return size;
	}

};

#endif /* CALTREE_H_ */
